(function() {
    "use strict";

    /**
     *  Creates the Spinner prototype
     *  Extends https://github.com/fgnass/spin.js v1.3.2
     *  @returns {Function} Spinner button prototype
     */

    function factory($, Spinner) {

        var defaults = {
            lines: 12,
            color: '#000',
            trail: 60,
            size: 'medium' // preset
        };

        var presets = {
            small: {
                length: 3.5,
                width: 1.5,
                radius: 3,
                corners: 2
            },
            medium: {
                length: 5.5,
                width: 2.25,
                radius: 5,
                corners: 6
            },
            large: {
                length: 8,
                width: 3,
                radius: 7,
                corners: 10
            }
        };

        /**
         *  Spinner's prototype with all the tm specific option presets
         *  @class
         *  @constructor
         *  @param {DIV} container a div container that will contain the spinner
         *  @param {Object} opts creation options
         */
        var SpinnerTM = function(container, opts) {

            container.each(function(){
                var $this = $(this);

                var options = $.extend({}, defaults, {color: $this.css('color')}, opts || {});

                var sizePresets = presets[options.size];
                if (!sizePresets){
                    throw new Error('Invalid tmSpinner size');
                }
                delete options.size;

                var data = $this.data();
                if (data.spinner){
                    data.spinner.stop();
                    delete data.spinner;
                }

                data.spinner = new Spinner($.extend({}, sizePresets, options)).spin(this);
            });

        };

        return SpinnerTM;
    }

    // If requirejs is present, we want to use it, otherwise, we want to use the global declarations to get the dependencies
    if (typeof define === 'function' && define.amd) {
        define(['jquery', 'spin'], factory);
    } else {
        tm.widgets.widgetFactory.make('tmSpinner', factory($, Spinner));
    }

})();